<?php

namespace Lizeo\PCPBundle\Service;

/**
 * Class SshConnector
 * @package Lizeo\PCPBundle\Service
 */
class SshConnector
{
    const
        HOST_KEY = 'ssh-rsa'
    ;

    /** @var string $sshHost */
    protected $sshHost;
    /** @var string $sshPort */
    protected $sshPort;
    /** @var string $sshUsername */
    protected $sshUsername;
    /** @var string $sshPrivateKeyPath */
    protected $sshPrivateKeyPath;
    /** @var string $sshPublicKeyPath */
    protected $sshPublicKeyPath;

    /** @var $session */
    protected $session;
    /** @var $connection */
    protected $connection;

    /**
     * @param $sshHost
     * @param $sshPort
     * @param $sshUsername
     * @param $sshPrivateKeyPath
     * @param $sshPublicKeyPath
     */
    public function __construct($sshHost, $sshPort, $sshUsername, $sshPrivateKeyPath, $sshPublicKeyPath)
    {
        $this
            ->setSshHost($sshHost)
            ->setSshPort($sshPort)
            ->setSshUsername($sshUsername)
            ->setSshPrivateKeyPath($sshPrivateKeyPath)
            ->setSshPublicKeyPath($sshPublicKeyPath)
            ->setConnection(false)
        ;

        return $this;
    }

    /**
     * @return $this
     * @throws \Exception
     */
    public function initSession() {
        $this->session = ssh2_connect(
            $this->getSshHost(), $this->getSshPort(), [
                'hostkey' => self::HOST_KEY
            ]
        );

        if (!$this->session)
        {
            throw new \Exception(
                sprintf(
                    "Unable to connect to SSH host %s@%s:%s with %s",
                    $this->getSshUsername(),
                    $this->getSshHost(),
                    $this->getSshPort(),
                    $this->getSshPublicKeyPath()
                )
            );
        }

        $this->connection = ssh2_auth_pubkey_file(
            $this->session,
            $this->getSshUsername(),
            $this->getSshPublicKeyPath(),
            $this->getSshPrivateKeyPath()
        );

        if (!$this->connection)
        {
            throw new \Exception(
                sprintf(
                    "SSH Authentication faillure with public-key on : %s@%s:%s with %s",
                    $this->getSshUsername(),
                    $this->getSshHost(),
                    $this->getSshPort(),
                    $this->getSshPublicKeyPath()
                )
            );
        }

        return $this;
    }

    /**
     * @return $this
     */
    public function closeSession() {
        $this
            ->setSession(null)
            ->setConnection(false);
        return $this;
    }

    /**
     * Permet de d'envoyer un fichier via SCP
     *
     * @param $localSrcPath
     * @param $remoteDestPath
     * @return bool
     * @throws \Exception
     */
    public function putFileByScp($localSrcPath, $remoteDestPath){
        if (!$this->session)
        {
            $this->initSession();
        }

        return ssh2_scp_send($this->session, $localSrcPath, $remoteDestPath);
    }

    /**
     * Permet de récupérer un fichier via SCP
     *
     * @param $remotePath
     * @param $localPath
     * @return bool
     * @throws \Exception
     */
    public function getFileByScp($remotePath, $localPath){
        if (!$this->session)
        {
            $this->initSession();
        }

        return ssh2_scp_recv($this->session, $remotePath, $localPath);
    }

    /**
     * Permet de lister la liste des fichiers dans un dossier
     *
     * @param null $path
     * @return array
     */
    public function getListByDirectory($path = null) {
        $files = [];
        $sftp = ssh2_sftp($this->session);

        $handle = opendir("ssh2.sftp://$sftp/". $path);
        while (false != ($entry = readdir($handle))) {
            if (in_array($entry, [ '.', '..' ])) continue; // Skip not interesting files
            $files[] = $entry;
        }
        return $files;
    }

    /**
     * @return string
     */
    public function getSshHost()
    {
        return $this->sshHost;
    }

    /**
     * @param string $sshHost
     * @return $this
     */
    public function setSshHost($sshHost)
    {
        $this->sshHost = $sshHost;
        return $this;
    }

    /**
     * @return string
     */
    public function getSshPort()
    {
        return $this->sshPort;
    }

    /**
     * @param string $sshPort
     * @return $this
     */
    public function setSshPort($sshPort)
    {
        $this->sshPort = $sshPort;
        return $this;
    }

    /**
     * @return string
     */
    public function getSshUsername()
    {
        return $this->sshUsername;
    }

    /**
     * @param string $sshUsername
     * @return $this
     */
    public function setSshUsername($sshUsername)
    {
        $this->sshUsername = $sshUsername;
        return $this;
    }

    /**
     * @return string
     */
    public function getSshPrivateKeyPath()
    {
        return $this->sshPrivateKeyPath;
    }

    /**
     * @param string $sshPrivateKeyPath
     * @return $this
     */
    public function setSshPrivateKeyPath($sshPrivateKeyPath)
    {
        $this->sshPrivateKeyPath = $sshPrivateKeyPath;
        return $this;
    }

    /**
     * @return string
     */
    public function getSshPublicKeyPath()
    {
        return $this->sshPublicKeyPath;
    }

    /**
     * @param string $sshPublicKeyPath
     * @return $this
     */
    public function setSshPublicKeyPath($sshPublicKeyPath)
    {
        $this->sshPublicKeyPath = $sshPublicKeyPath;
        return $this;
    }

    /**
     * @return bool
     */
    public function getConnection()
    {
        return $this->connection;
    }

    /**
     * @param bool $connection
     * @return $this
     */
    public function setConnection($connection)
    {
        $this->connection = $connection;
        return $this;
    }

    /**
     * @return mixed
     */
    public function getSession()
    {
        return $this->session;
    }

    /**
     * @param $session
     * @return $this
     */
    public function setSession($session)
    {
        $this->session = $session;
        return $this;
    }
}
